<?php
  ########################################################################
  # Original Resume Download Class © Kosinix, https://github.com/kosinix #
  # Original Code : https://gist.github.com/kosinix/4cf0d432638817888149 #
  # Edited and Improved By Alessandro Marinuzzi • https://www.alecos.it/ #
  # RT Download Counter 2.4 • 17/12/2022 © Alessandro Marinuzzi [Alecos] #
  ########################################################################
  include($_SERVER["DOCUMENT_ROOT"] . "/php/config.php");
  if (!defined('PHP_EOL')) {
    if (strtoupper(substr(PHP_OS,0,3) == 'WIN')) {
      define('PHP_EOL',"\r\n");
    } elseif (strtoupper(substr(PHP_OS,0,3) == 'MAC')) {
      define('PHP_EOL',"\r");
    } elseif (strtoupper(substr(PHP_OS,0,3) == 'DAR')) {
      define('PHP_EOL',"\n");
    } else {
      define('PHP_EOL',"\n");
    }
  }
  class ResumeDownload {
    private $file;
    private $name;
    private $boundary;
    private $delay = 0;
    private $size = 0;
    function __construct($file, $delay = 0) {
      if (!is_file($file)) {
        header("HTTP/1.1 400 Invalid Request");
        exit('File Not Found');
      }
      $this->size = filesize($file);
      $this->file = fopen($file, "rb");
      $this->boundary = md5($file);
      $this->delay = $delay;
      $this->name = basename($file);
    }
    public function process() {
      $ranges = NULL;
      $t = 0;
      if ($_SERVER['REQUEST_METHOD'] == 'GET' && isset($_SERVER['HTTP_RANGE']) && $range = stristr(trim($_SERVER['HTTP_RANGE']), 'bytes=')) {
        $range = substr($range, 6);
        $ranges = explode(',', $range);
        $t = count($ranges);
      }
      header("Accept-Ranges: bytes");
      header("Content-Type: application/octet-stream");
      header("Content-Transfer-Encoding: binary");
      header(sprintf('Content-Disposition: attachment; filename="%s"', $this->name));
      if ($t > 0) {
        header("HTTP/1.1 206 Partial Content");
        $t === 1 ? $this->pushSingle($range) : $this->pushMulti($ranges);
      } else {
        header("Content-Length: " . $this->size);
        $this->readFile();
      }
      ob_flush();
      flush();
    }
    private function pushSingle($range) {
      $start = $end = 0;
      $this->getRange($range, $start, $end);
      header("Content-Length: " . ($end - $start + 1));
      header(sprintf("Content-Range: bytes %d-%d/%d", $start, $end, $this->size));
      fseek($this->file, $start);
      $this->readBuffer($end - $start + 1);
      $this->readFile();
    }
    private function pushMulti($ranges) {
      $length = $start = $end = 0;
      $output = "";
      $tl = "Content-Type: application/octet-stream" . PHP_EOL;
      $formatRange = "Content-Range: bytes %d-%d/%d" . PHP_EOL . PHP_EOL;
      foreach ($ranges as $range) {
        $this->getRange($range, $start, $end);
        $length += strlen(PHP_EOL . "--$this->boundary" . PHP_EOL);
        $length += strlen($tl);
        $length += strlen(sprintf($formatRange, $start, $end, $this->size));
        $length += $end - $start + 1;
      }
      $length += strlen(PHP_EOL . "--$this->boundary--" . PHP_EOL);
      header("Content-Length: $length");
      header("Content-Type: multipart/x-byteranges; boundary=$this->boundary");
      foreach ($ranges as $range) {
        $this->getRange($range, $start, $end);
        echo PHP_EOL . "--$this->boundary" . PHP_EOL;
        echo $tl;
        echo sprintf($formatRange, $start, $end, $this->size);
        fseek($this->file, $start);
        $this->readBuffer($end - $start + 1);
      }
      echo PHP_EOL . "--$this->boundary--" . PHP_EOL;
    }
    private function getRange($range, &$start, &$end) {
      list ($start, $end) = explode('-', $range);
      $fileSize = $this->size;
      if ($start == '') {
        $tmp = $end;
        $end = $fileSize - 1;
        $start = $fileSize - $tmp;
        if ($start < 0) {
          $start = 0;
        }
      } else {
        if ($end == '' || $end > $fileSize - 1) {
          $end = $fileSize - 1;
        }
      }
      if ($start > $end) {
        header("Status: 416 Requested Range Not Satisfiable");
        header("Content-Range: */" . $fileSize);
        exit();
      }
      return array($start, $end);
    }
    private function readFile() {
      while (!feof($this->file)) {
        echo fgets($this->file);
        ob_flush();
        flush();
        usleep($this->delay);
      }
    }
    private function readBuffer($bytes, $size = 1024) {
      $bytesLeft = $bytes;
      while ($bytesLeft > 0 && !feof($this->file)) {
        $bytesLeft > $size ? $bytesRead = $size : $bytesRead = $bytesLeft;
        $bytesLeft -= $bytesRead;
        echo fread($this->file, $bytesRead);
        ob_flush();
        flush();
        usleep($this->delay);
      }
    }
  }
?>